package javax.swing.colorchooser;

import javax.swing.SwingUtilities;
import java.awt.event.FocusEvent;
import java.text.ParseException;
import java.util.Locale;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.DocumentFilter;
import java.awt.event.FocusListener;
import javax.swing.JFormattedTextField;

final class ValueFormatter extends JFormattedTextField.AbstractFormatter implements FocusListener, Runnable
{
    private final DocumentFilter filter;
    private final int length;
    private final int radix;
    private JFormattedTextField text;
    
    static void init(int columns, boolean b, JFormattedTextField formattedTextField) {
        ValueFormatter valueFormatter = new ValueFormatter(columns, b);
        formattedTextField.setColumns(columns);
        formattedTextField.setFormatterFactory(new DefaultFormatterFactory(valueFormatter));
        formattedTextField.setHorizontalAlignment(4);
        formattedTextField.setMinimumSize(formattedTextField.getPreferredSize());
        formattedTextField.addFocusListener(valueFormatter);
    }
    
    ValueFormatter(int length, boolean b) {
        this.filter = new DocumentFilter() {
            @Override
            public void remove(DocumentFilter.FilterBypass filterBypass, int n, int n2) throws BadLocationException {
                if (ValueFormatter.access$000(ValueFormatter.this, filterBypass.getDocument().getLength() - n2)) {
                    filterBypass.remove(n, n2);
                }
            }
            
            @Override
            public void replace(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) throws BadLocationException {
                if (ValueFormatter.access$000(ValueFormatter.this, filterBypass.getDocument().getLength() + s.length() - n2) && ValueFormatter.access$100(ValueFormatter.this, s)) {
                    filterBypass.replace(n, n2, s.toUpperCase(Locale.ENGLISH), set);
                }
            }
            
            @Override
            public void insertString(DocumentFilter.FilterBypass filterBypass, int n, String s, AttributeSet set) throws BadLocationException {
                if (ValueFormatter.access$000(ValueFormatter.this, filterBypass.getDocument().getLength() + s.length()) && ValueFormatter.access$100(ValueFormatter.this, s)) {
                    filterBypass.insertString(n, s.toUpperCase(Locale.ENGLISH), set);
                }
            }
        };
        this.length = length;
        this.radix = (b ? 16 : 10);
    }
    
    @Override
    public Object stringToValue(String s) throws ParseException {
        try {
            return Integer.valueOf(s, this.radix);
        }
        catch (NumberFormatException cause) {
            ParseException ex = new ParseException("illegal format", 0);
            ex.initCause(cause);
            throw ex;
        }
    }
    
    @Override
    public String valueToString(Object o) throws ParseException {
        if ((o instanceof Integer)) {
            if (this.radix == 10) {
                return o.toString();
            }
            int intValue = (int)o;
            int length = this.length;
            char[] value = new char[length];
            while (0 < length--) {
                value[length] = Character.forDigit(intValue & 0xF, this.radix);
                intValue >>= 4;
            }
            return new String(value).toUpperCase(Locale.ENGLISH);
        }
        else {
            throw new ParseException("illegal object", 0);
        }
    }
    
    @Override
    protected DocumentFilter getDocumentFilter() {
        return this.filter;
    }
    
    @Override
    public void focusGained(FocusEvent focusEvent) {
        Object source = focusEvent.getSource();
        if ((source instanceof JFormattedTextField)) {
            this.text = (JFormattedTextField)source;
            SwingUtilities.invokeLater(this);
        }
    }
    
    @Override
    public void focusLost(FocusEvent focusEvent) {
    }
    
    @Override
    public void run() {
        if (this.text != null) {
            this.text.selectAll();
        }
    }
    
    private boolean isValid(int n) {
        return 0 <= n && n <= this.length;
    }
    
    private boolean isValid(String s) {
        int length = s.length();
        int i = 0;
        while (i < length) {
            if (Character.digit(s.charAt(i), this.radix) < 0) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    static boolean access$000(ValueFormatter valueFormatter, int n) {
        return valueFormatter.isValid(n);
    }
    
    static boolean access$100(ValueFormatter valueFormatter, String s) {
        return valueFormatter.isValid(s);
    }
}
